/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Copyright: 2012 Labo A.L
 * Authors:
 *   Aaron Lewis <the.warl0ck.1989@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */
#include "mainwindow.h"
#include "ui_mainwindow.h"

#define HIDE_TRANSCRIPTS(ok) do { \
    ui->transcript->setVisible(ok); \
    ui->lineWidget->setVisible(ok); \
    } while (0);

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow),
    m_vol (80),
    m_filterModel (new QSortFilterProxyModel (this)),
    m_model (new QStandardItemModel (this))
{
    ui->setupUi(this);
    ui->centralWidget->setCurrentIndex(0);

    QAction *copyURL_action = new QAction (tr("Copy video URL"), this);
    connect (copyURL_action, SIGNAL(triggered()), this, SLOT(copyDownloadURL()));
    ui->programeListView->addAction(copyURL_action);

    HIDE_TRANSCRIPTS(false);

    ui->programeListView->setModel(m_filterModel);
    m_filterModel->setSourceModel(m_model);
    m_filterModel->setFilterCaseSensitivity(Qt::CaseInsensitive);

    ui->statusLabel->setMovie(new QMovie (":/images/loading.gif"));
    ui->statusLabel->movie()->start();

    ui->mplayer->setSeekSlider(ui->seekSlider);
    ui->mplayer->setVolumeSlider(ui->volumeSlider);
    ui->mplayer->setStatusBar(ui->statusBar);

    connect (ui->mplayer, SIGNAL(stateChanged(int)) , this, SLOT(mplayerStateChanged(int)));
    connect (ui->volumeSlider, SIGNAL(sliderPressed()), this, SLOT(update_volume_icon()));
    update_volume_icon();

    m_providers.append(new TedProvider (m_model, this));
    m_providers.append(new SciAmericanProvider (m_model, this));

    foreach (Provider *p, m_providers)
    {
        connect (p, SIGNAL(loaded()), this, SLOT(loadCompleted()));
    }

    ///
    ui->actionRefresh->trigger();
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::copyDownloadURL()
{
    QModelIndex idx = ui->programeListView->currentIndex();
    if ( idx.isValid() )
    {
        QClipboard *clip = QApplication::clipboard();
        clip->setText(m_filterModel->data(idx, Qt::UserRole).toString());
    }
}

void MainWindow::loadCompleted()
{
    ++ m_completed_cnt;
    if ( m_completed_cnt == m_providers.size() )
    {
        ui->statusLabel->hide();
    }
}

void MainWindow::on_stopButton_clicked()
{
    ui->mplayer->stop();
}

void MainWindow::on_playPauseButton_clicked()
{
    if ( ui->mplayer->state() == QMPwidget::NotStartedState)
        ui->mplayer->start();

    if ( ui->mplayer->state() == QMPwidget::PlayingState )
    {
        ui->mplayer->pause();
    }
    else
    {
        ui->mplayer->play();
    }
}

void MainWindow::mplayerStateChanged(int s)
{
    if ( s == QMPwidget::PlayingState )
    {
        ui->playPauseButton->setIcon(QIcon(":/images/media-playback-pause.png"));
    }
    else
    {
        ui->playPauseButton->setIcon(QIcon(":/images/media-playback-start.png"));
    }
}

void MainWindow::on_muteButton_toggled(bool checked)
{
    if ( checked )
    {
        m_vol = ui->volumeSlider->value();
        ui->muteButton->setIcon(QIcon(":/images/audio-volume-muted.png"));
        ui->volumeSlider->setValue(0);
    }
    else
    {
        ui->volumeSlider->setValue(m_vol);
        update_volume_icon();
    }
}

void MainWindow::update_volume_icon()
{
    m_vol = ui->volumeSlider->value();
    if ( m_vol > 80 )
    {
        ui->muteButton->setIcon(QIcon(":/images/audio-volume-high.png"));
    }
    else if ( m_vol > 50 )
    {
        ui->muteButton->setIcon(QIcon(":/images/audio-volume-medium.png"));
    }
    else if ( m_vol > 0 )
    {
        ui->muteButton->setIcon(QIcon(":/images/audio-volume-low.png"));
    }
    else
    {
        ui->muteButton->setIcon(QIcon(":/images/audio-volume-muted.png"));
    }
}

void MainWindow::on_actionRefresh_triggered()
{
    m_completed_cnt = 0;
    ui->statusLabel->show();
    m_model->clear();

    foreach (Provider *p, m_providers)
    {
        p->start();
    }
}

void MainWindow::on_actionToggleView_triggered(bool checked)
{
    ui->centralWidget->setCurrentIndex( checked ? 1 : 0 );
}

void MainWindow::closeEvent(QCloseEvent *)
{
    ui->mplayer->stop();
}

void MainWindow::on_programeListView_doubleClicked(const QModelIndex &index)
{
    if ( index.isValid() )
    {
        const QMap<int,QVariant> mapping = m_filterModel->itemData(index);
        if ( mapping.contains(Qt::UserRole + OFFSET_TRANSCRIPT) )
        {
            HIDE_TRANSCRIPTS(true);
            ui->transcript->setHtml(mapping.value(Qt::UserRole + OFFSET_TRANSCRIPT).toString());
        }
        else
            HIDE_TRANSCRIPTS(false);

        ui->mplayer->setVisible( ui->transcript->isHidden() );

        ui->actionToggleView->trigger();

        if ( ui->mplayer->state() == QMPwidget::NotStartedState)
            ui->mplayer->start();

        ui->mplayer->load(mapping.value(Qt::UserRole).toString());
    }
}

void MainWindow::on_subtitleFilter_textChanged(const QString &arg1)
{
    m_filterModel->setFilterWildcard(arg1);
}

void MainWindow::on_searchTextLine_textChanged(const QString &arg1)
{
    Q_UNUSED (arg1)
    ui->transcript->moveCursor(QTextCursor::Start);
    on_lookForward_clicked();
}

void MainWindow::on_lookForward_clicked()
{
    ui->transcript->find(ui->searchTextLine->text());
}

void MainWindow::on_lookBackward_clicked()
{
    ui->transcript->find(ui->searchTextLine->text(), QTextDocument::FindBackward);
}

void MainWindow::on_searchTextLine_returnPressed()
{
    on_lookForward_clicked();
}

void MainWindow::keyPressEvent(QKeyEvent *e)
{
    switch (e->key())
    {
    case Qt::Key_F6:
        QLineEdit *lineEdit = ui->searchTextLine;
        if ( ui->centralWidget->currentIndex() == 0 )
            lineEdit = ui->subtitleFilter;
        lineEdit->setFocus();
        lineEdit->selectAll();
        break;
    }
}

void MainWindow::on_actionAbout_Qt_triggered()
{
    QMessageBox::aboutQt(this);
}

void MainWindow::on_actionAbout_Me_triggered()
{
    QMessageBox::about(this, tr("Minicast"),
                       tr("Contact: <b>the.warl0ck.1989@gmail.com</b><br/>"
                          "Access <b>blogspot.c0debreak.com</b> for more information "
                          "(note the zero from both, not a big char O)"));
}

void MainWindow::on_actionQuit_Q_triggered()
{
    QApplication::quit();
}
